# -*- coding: utf-8 -*-

"""
"""

__author__ = "Stefan Hendricks"

import requests
from dateutil.parser import parse as dtparse
from datetime import datetime, timedelta
from pathlib import Path
from loguru import logger
import numpy as np

from icedrift import GeoReferenceStation


class PolarsternAWIDashboardPos(object):

    def __init__(self, begin_date, end_date, time_pad_minutes=10):
        """
        Get Polarstern position and heading from the Polarstern HYDRINS1 sensor
        NOTE: The URL etc can be checked using the web interface at
              https://dashboard.awi.de/data-xxl/index.html
        :param begin_date: (datetime.datetime) Start of the requested temporal coverage
        :param end_date: (datetime.datetime)Start of the requested temporal coverage
        :param time_pad_minutes: (integer) The number of minutes the period will be extended
            on both side to ensure the data coverage of Polarstern position and heading
            data always fully comprised the requested data period

        """

        # Save input arguments and automatically extend data period
        # using a short time window to ensure that there is always
        # Polarstern data before the requested data period.
        self.begin_date = begin_date - timedelta(minutes=time_pad_minutes)
        self.end_date = end_date + timedelta(minutes=time_pad_minutes)
        # --- Properties ---

        # The format of the REST output.
        self.requested_format = "text/csv",

        # Output format for the time stamp
        # NOTE: Most sensors do not report faster than 1 second, therefore there is no need
        #       for microseconds
        self.datetime_format = "%Y-%m-%dT%H:%M:%S"

        # The main URL to the dashboard REST interface
        self.base_url = "https://dashboard.awi.de/data-xxl/rest/data"

        # Parameter to be pulled from the Polarstern Inertial Navigation Unit
        # 1. Latitude
        # 2. Longitude
        # 3. True Heading
        self.sensors = ["vessel:polarstern:hydrins_1:latitude$NRT",
                        "sensors=vessel:polarstern:hydrins_1:longitude$NRT",
                        "sensors=vessel:polarstern:hydrins_1:heading$NRT"]

        # Temporal resolution (full)
        self.aggregate_value = "second"

        # Load the data
        self.response = None
        self.download()

        self._variables = {}
        self.decode_response()

    def download(self):
        """
        Pulls the data for the requested period from dashboard.awi.de
        :return:
        """

        # Construct the URL
        url = self._construct_url()
        logger.info("data request url: {}".format(url))

        # Download the websites response
        # TODO: Error management
        r = requests.get(url)

        # Post-process the content
        self.response = "\n".join(r.text.splitlines())
        self.response = self.response.replace(";", ",")

    def decode_response(self):
        """
        Parses the response from the data service into arrays of time, longitude, latitude
        and true heading.
        :return:
        """

        # Init the variables
        # NOTE: the first line is the header line and has to be ignored.
        lines = self.response.split("\n")[1:]
        n_lines = len(lines)
        self._variables["time"] = np.full(n_lines, object)
        for parameter_name in ["longitude", "latitude", "heading"]:
            self._variables[parameter_name] = np.full(n_lines, np.nan)

        for i, line in enumerate(lines):
            string_array = line.split(",")
            self._variables["time"][i] = dtparse(string_array[0])
            self._variables["latitude"][i] = np.float(string_array[1])
            self._variables["longitude"][i] = np.float(string_array[2])
            self._variables["heading"][i] = np.float(string_array[3])

    def save(self, filename):
        """
        :param filename:
        :return:
        """
        raise NotImplementedError()

    def _construct_url(self):
        """
        Create the url from the input
        :return:
        """

        # Add base url
        url = self.base_url+"?"

        # add sensors
        url += self.sensor_def

        # add output format
        url += "&format=%s" % self.requested_format

        # add temporal resolution
        url += "&aggregate={}".format(self.aggregate_value)

        # add begin date (optional)
        if self.begin_date is not None:
            url += "&beginDate={}".format(self.begin_date.strftime(self.datetime_format))

        # add end date (optional)
        if self.end_date is not None:
            url += "&endDate={}".format(self.end_date.strftime(self.datetime_format))

        return url

    @property
    def sensor_def(self):
        """
        Constructs the sensor string from the list of sensors
        :return: (str) the sensor string in the form of sensors={...}&{...}&{...}
        """
        sensor_def = "sensors="+"&".join(self.sensors)
        return sensor_def

    @property
    def reference_station(self):
        """
        Return a icedrift.GeoReferenceStation instance
        :return:
        """
        refstat = GeoReferenceStation(self.time, self.longitude, self.latitude, self.heading)
        return refstat

    def __getattr__(self, attr):
        """
        Modify the attribute getter to provide a shortcut to the data content
        :param attr:
        :return:
        """
        if attr in list(self._variables.keys()):
            return self._variables[attr]
        else:
            raise AttributeError()


class PolarsternPOSDashboardData(object):
    """
    Interface to Polarstern data repository downloaded from the dashboard
    REST interface.
    """

    def __init__(self, repo_folder, filename_template="ps-pos-{year}-{month:02g}-{day:02g}.csv"):
        """

        :param repo_folder:
        :return:
        """

        # --- Properties ---

        # Lookup directory for the data files
        self.repo_folder = repo_folder

        # Filename template
        self.filename_template = filename_template

    def get_date(self, target_date):
        """
        Get a position object for a single (UTC) day
        :param target_date: datetime.date
        :return:
        """

        # Construct the file path of the target (daily) file
        filepath = self._get_filepath(target_date)

        # Read the file
        data = self._get_file_content(filepath)
        return data

    def _get_filepath(self, date):
        """
        Return the filename for a date
        :param date: datetime.date
        :return: pathlib.Path
        """
        filename = self.filename_template.format(year=date.year, month=date.month, day=date.day)
        filepath = Path(self.repo_folder) / filename
        return filepath

    @staticmethod
    def _get_file_content(filepath):
        """
        Read a file and return the data as position object
        :param filepath:
        :return:
        """

        # Converter of datetime strings to datetime object
        str2date = lambda x: datetime.strptime(x.decode("utf-8"), '%Y-%m-%dT%H:%M:%S.000')

        # Read csv content
        content = np.genfromtxt(filepath, delimiter=',', skip_header=1,
                                names="time, latitude, longitude, heading",
                                dtype=(object, float, float, float),
                                converters={0: str2date})

        # Create a georeference station
        data = GeoReferenceStation(content["time"], content["longitude"], content["latitude"], content["heading"])

        return data
